# Generated by roxygen2: do not edit by hand

S3method(geo_bb,bbox)
S3method(geo_bb,matrix)
S3method(geo_bb,sf)
S3method(geo_bb_matrix,matrix)
S3method(geo_bb_matrix,numeric)
S3method(geo_bb_matrix,sf)
S3method(geo_buffer,sf)
S3method(geo_buffer,sfc)
S3method(geo_length,sf)
S3method(geo_projected,sf)
S3method(geo_projected,sfc)
S3method(geo_select_aeq,sf)
S3method(geo_select_aeq,sfc)
S3method(gsection,sf)
S3method(islines,sf)
S3method(line2df,sf)
S3method(line2points,sf)
S3method(line2points,sfc)
S3method(line2points,sfg)
S3method(line2pointsn,sf)
S3method(line2vertices,sf)
S3method(line_segment,sf)
S3method(line_segment,sfc_LINESTRING)
S3method(line_segment1,sf)
S3method(line_segment1,sfc_LINESTRING)
S3method(n_vertices,sf)
S3method(od2line,sf)
S3method(onewaygeo,sf)
S3method(overline,sf)
S3method(points2line,sf)
S3method(points2odf,Spatial)
S3method(points2odf,sf)
S3method(rnet_group,default)
S3method(rnet_group,sf)
S3method(rnet_group,sfc)
S3method(route,character)
S3method(route,numeric)
S3method(route,sf)
export("%>%")
export(angle_diff)
export(bb2poly)
export(bbox_scale)
export(geo_bb)
export(geo_bb_matrix)
export(geo_buffer)
export(geo_code)
export(geo_length)
export(geo_projected)
export(geo_select_aeq)
export(geo_toptail)
export(gsection)
export(is_linepoint)
export(islines)
export(line2df)
export(line2points)
export(line2pointsn)
export(line2vertices)
export(line_bearing)
export(line_breakup)
export(line_cast)
export(line_midpoint)
export(line_segment)
export(line_segment1)
export(line_via)
export(mats2line)
export(n_segments)
export(n_vertices)
export(od2line)
export(od2odf)
export(od_aggregate_from)
export(od_aggregate_to)
export(od_coords)
export(od_coords2line)
export(od_id_character)
export(od_id_max_min)
export(od_id_order)
export(od_id_szudzik)
export(od_oneway)
export(od_to_odmatrix)
export(odmatrix_to_od)
export(onewaygeo)
export(overline)
export(overline2)
export(overline_intersection)
export(points2flow)
export(points2line)
export(points2odf)
export(quadrant)
export(read_table_builder)
export(rnet_add_node)
export(rnet_boundary_df)
export(rnet_boundary_points)
export(rnet_boundary_points_lwgeom)
export(rnet_boundary_unique)
export(rnet_breakup_vertices)
export(rnet_connected)
export(rnet_duplicated_vertices)
export(rnet_get_nodes)
export(rnet_group)
export(rnet_join)
export(rnet_merge)
export(rnet_subset)
export(route)
export(route_average_gradient)
export(route_dodgr)
export(route_google)
export(route_nearest_point)
export(route_osrm)
export(route_rolling_average)
export(route_rolling_diff)
export(route_rolling_gradient)
export(route_sequential_dist)
export(route_slope_matrix)
export(route_slope_vector)
export(route_split)
export(route_split_id)
export(toptail_buff)
import(curl)
importFrom(Rcpp,evalCpp)
importFrom(dplyr,first)
importFrom(dplyr,last)
importFrom(dplyr,n)
importFrom(graphics,text)
importFrom(magrittr,"%>%")
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(rlang,.data)
importFrom(stats,setNames)
importFrom(utils,download.file)
importFrom(utils,read.csv)
importFrom(utils,tail)
importFrom(utils,unzip)
